
#ifndef ENDIAN_H
#define ENDIAN_H

#include <iostream>

//Is this architecture little endian?
inline
bool IsLittleEndian(){
	long l=1;
	return ((*(char*)&l));
}

//Is this architecture big endian?
inline
bool IsBigEndian(){
	return !IsLittleEndian();
}

//Old, incorrect implementation. 
/*inline
void ByteSwap(char* b, int n){
	int i=0;
	int j=n-1;
	while(i,j){
		std::swap(b[i],b[j]);
			i++; j--;
	}
}*/

inline
void ByteSwap(char* b, int n){
	for (int i = 0; i <  n/2; i++) {
		std::swap(b[i], b[n-1-i]);
	}
}

inline
void ReadFromBigEndian(char* ptr, int nelem, int bytesPerElem, std::istream &is){
		is.read(ptr, nelem*bytesPerElem);
		if(IsLittleEndian()){
			//Swap byte order
			for(int i= 0; i<nelem*bytesPerElem; i+=bytesPerElem){
				ByteSwap(&ptr[i],bytesPerElem);	
			}
		}
}

inline
void WriteToBigEndian(char* ptr, int nelem, int bytesPerElem, std::ostream &os){
	if(IsBigEndian()){
		os.write(ptr, nelem*bytesPerElem);
	}
	else{
		//Swap byte order
		for(int i= 0; i<nelem*bytesPerElem; i+=bytesPerElem){
			ByteSwap(&ptr[i],bytesPerElem);	
		}

		os.write(ptr, nelem*bytesPerElem);

		//Restore byte order
		for(int i= 0; i<nelem*bytesPerElem; i+=bytesPerElem){
			ByteSwap(&ptr[i],bytesPerElem);	
		}
	}
}

inline
void ReadFromLittleEndian(char* ptr, int nelem, int bytesPerElem, std::istream &is){
		is.read(ptr, nelem*bytesPerElem);
		if(IsBigEndian()){
			//Swap byte order
			for(int i= 0; i<nelem*bytesPerElem; i+=bytesPerElem){
				ByteSwap(&ptr[i],bytesPerElem);	
			}
		}
}

inline
void WriteToLittleEndian(char* ptr, int nelem, int bytesPerElem, std::ostream &os){
	if(IsLittleEndian()){
		os.write(ptr, nelem*bytesPerElem);
	}
	else{
		//Swap byte order
		for(int i= 0; i<nelem*bytesPerElem; i+=bytesPerElem){
			ByteSwap(&ptr[i],bytesPerElem);	
		}

		os.write(ptr, nelem*bytesPerElem);

		//Restore byte order
		for(int i= 0; i<nelem*bytesPerElem; i+=bytesPerElem){
			ByteSwap(&ptr[i],bytesPerElem);	
		}
	}
}



#endif