#ifndef _FILENAME_H
#define _FILENAME_H

#include <vector>
#include <string>
using namespace std;

inline vector<string> StringSplit(const string &source, const char* delimiter=" "){
	vector<string> results;

	size_t prev = 0;
	size_t next = 0;

	while(next!=string::npos){
		next = source.find_first_of(delimiter, prev);
		if (next - prev != 0){
			results.push_back(source.substr(prev, next - prev));
		}
		prev = next + 1;
	}

	return results;
}

//A class for handling filenames on Windows
class FileName{
public:
	FileName(){}
	FileName(std::string fullPath){
		path =StringSplit(fullPath,"\\/");

		if(!(fullPath.back()=='\\' || fullPath.back()=='/')){
			string tmp=path.back();
			path.pop_back();
			size_t ext_delimiter=tmp.find_last_of (".");
			name=tmp.substr(0,ext_delimiter);
			if(ext_delimiter!=string::npos){
				extension=tmp.substr(ext_delimiter+1);
			}
		}
	}

	std::string GetAsString(){string s;
	for(int i=0; i<path.size(); i++){
		s.append(path[i]);
		s.append("\\");
	}
	s.append(name);
	s.append(".");
	s.append(extension);

	return s;
	}

	std::string GetName(){return name;}
	std::string GetExtension(){return extension;}

	void SetName(std::string s){name=s;}
	void SetExtension(std::string s){extension=s;}

	void ClearPath(){path.clear();}
	void PathAppend(std::string s){path.push_back(s);}

	void PathPrepend(std::string s){
		vector<string> new_path(path.size()+1);
		new_path[0]=s;
		for(int i=0; i<path.size(); i++){
			new_path[i+1]=path[i];
		}
		path=new_path;
	}

private:
	std::vector<std::string > path;
	std::string name;
	std::string extension;
};

#endif